#include <syslog.h>
#include <string.h>
#include "button.h"
#include "log.h"
#include "server.h"
#include "gpio.h"

void gpio_button_init(struct server_ctx *s_ctx);

typedef enum {
	MODE_UNKNOWN,
	DIRECT,
	SHIFTREG_BRCM,
	SHIFTREG_GPIO,
	GPIO_LINUX,
} gpio_mode_t;

struct gpio_button_data {
	int addr;
	gpio_mode_t mode;
	int active;
	struct button_drv button;
};

static button_state_t gpio_button_get_state(struct button_drv *drv)
{
//	DBG(1, "state for %s",  drv->name);
	struct gpio_button_data *p = (struct gpio_button_data *)drv->priv;
	int value;

#ifdef BRCM_BOARD
	value = board_ioctl( BOARD_IOCTL_GET_GPIO, 0, 0, NULL, p->addr, 0);
#else
	value = gpio_linux_get(p->addr);
#endif
	if(p->active)
		return !!value;
	else
		return !value;
}

static struct button_drv_func func = {
	.get_state = gpio_button_get_state,
};

void gpio_button_init(struct server_ctx *s_ctx) {
	struct ucilist *node;
	LIST_HEAD(buttons);

	DBG(1, "");

	gpio_init();

	ucix_get_option_list(s_ctx->uci_ctx, "hw" ,"gpio_buttons", "buttons", &buttons);
	list_for_each_entry(node, &buttons, list) {
		struct gpio_button_data *data;
		const char *s;
		int tmp;

		DBG(1, "value = [%s]",node->val);

		data = malloc(sizeof(struct gpio_button_data));
		memset(data,0,sizeof(struct gpio_button_data));

		data->button.name = node->val;

		s = ucix_get_option(s_ctx->uci_ctx, "hw" , data->button.name, "mode");
		DBG(1, "mode = [%s]", s);
		if (s) {

			if (!strncasecmp("direct",s,6))
				data->mode =  DIRECT;
			else if (!strncasecmp("linux",s,5))
				data->mode =  GPIO_LINUX;
			else if (!strncasecmp("sr",s,2))
				data->mode =  SHIFTREG_BRCM;
			else if (!strncasecmp("csr",s,3))
				data->mode =  SHIFTREG_GPIO;
			else
				DBG(1, "Mode %s : Not supported!", s);
		}

		s = ucix_get_option(s_ctx->uci_ctx, "hw" , data->button.name, "addr");
		DBG(1, "addr = [%s]", s);

		if (s){
			data->addr =  strtol(s,0,0);
		}

		gpio_linux_input_init(data->addr);

		/* Query gpio to setup port and supress possible ghost presses
		 * Workaround for some issue in the broadcom gpio driver */
		gpio_get_state(data->addr);
		gpio_get_state(data->addr);
		gpio_get_state(data->addr);

		s = ucix_get_option(s_ctx->uci_ctx, "hw" , data->button.name, "active");
		data->active = -1;
		if (s){
			if (!strncasecmp("hi",s,2))
				data->active =  1;
			else if (!strncasecmp("low",s,3))
				data->active =  0;

		}
		DBG(1, "active = %d", data->active);

		data->button.func = &func;
		data->button.priv = data;

		button_add(&data->button);
	}
}
